<?php

namespace BS\RealTimeChat\DevelopmentOutput;

use XF\Mvc\Entity\Entity;
use XF\Util\Json;
use XF\DevelopmentOutput\AbstractHandler;

class Command extends AbstractHandler
{
    protected function getTypeDir()
    {
        return 'bs_rtc_command';
    }

    public function export(Entity $command)
    {
        if (!$this->isRelevant($command))
        {
            return true;
        }

        $fileName = $this->getFileName($command);

        $json = [
            'command_id' => $command->command_id,
            'command_class' => $command->command_class,
            'command_regex' => $command->command_regex,
            'active' => $command->active
        ];

        return $this->developmentOutput->writeFile($this->getTypeDir(), $command->addon_id, $fileName, Json::jsonEncodePretty($json));
    }

    protected function getEntityForImport($name, $addOnId, $json, array $options)
    {
        $command = \XF::em()->getFinder('BS\RealTimeChat:Command')->where([
            'command_id' => $json->command_id
        ])->fetchOne();
        if (!$command)
        {
            $command = \XF::em()->create('BS\RealTimeChat:Command');
        }

        $command = $this->prepareEntityForImport($command, $options);

        return $command;
    }

    public function import($name, $addOnId, $contents, array $metadata, array $options = [])
    {
        $json = json_decode($contents, true);

        $command = $this->getEntityForImport($name, $addOnId, $json, $options);

        $command->bulkSet($json);
        $command->addon_id = $addOnId;
        $command->save();

        return $command;
    }

    protected function getFileName(Entity $command, $new = true)
    {
        $commandId = $new ? $command->getValue('command_id') : $command->getExistingValue('command_id');

        return "{$commandId}.json";
    }
}