<?php

namespace BS\RealTimeChat;

use BS\RealTimeChat\LongPoll\Communicator;
use BS\RealTimeChat\LongPoll\Room\Db;
use XF\BbCode\RuleSet;
use XF\Mvc\Entity\Entity;

class Listener
{
    public static function renderCopyright()
    {
        if (\XF::config('bsRemoveCopyright'))
        {
            return '';
        }

        $request = \XF::app()->request();
        if (! ($domain = $request->getServer('HTTP_HOST')))
        {
            $boardUrl = \XF::options()->boardUrl;
            $urlParts = parse_url($boardUrl);

            if ($urlParts['host'])
            {
                $domain = $urlParts['host'];
            }
            else
            {
                $domain = $boardUrl;
            }
        }

        return '<a href="https://devsell.io/?utm_source=' . urlencode($domain) . '&utm_medium=chat_addon&utm_campaign=product_branding" target="_blank" rel="nofollow" class="chat-copyright">Add-on by 021</a>';
    }

    public static function appSetup(\XF\App $app)
    {
        $app['longPollingCommunicator'] = function (\XF\Container $c) use ($app)
        {
            return new Communicator($app);
        };

        $app['chatEmitter'] = function (\XF\Container $c) use ($app)
        {
            if ($app->options()->realTimeChatClientCommunication['communication'] == 'long_polling')
            {
                return $app['longPollingCommunicator'];
            }

            return new \Emitter('127.0.0.1', $c['options']->realTimeChatChannelAdapterPort);
        };
    }

    public static function entityStructureUser(\XF\Mvc\Entity\Manager $em, \XF\Mvc\Entity\Structure &$structure)
    {
        $structure->relations += [
            'ChatBan' => [
                'entity' => 'BS\RealTimeChat:Banned',
                'type' => Entity::TO_ONE,
                'conditions' => 'user_id',
                'primary' => false
            ]
        ];
    }

    public static function entityPostDeleteUser(Entity $user)
    {
        $user->db()->delete('xf_bs_chat', 'user_id = ?', $user->user_id);
    }

    public static function entityPostSaveUserProfile(Entity $profile)
    {
        if ($profile->isChanged('ignored'))
        {
            $emitter = \XF::app()['chatEmitter'];
            $emitter->in($profile->user_id)->emit('update ignore', array_keys($profile->ignored));
        }
    }

    public static function bbCodeRules(RuleSet $ruleSet, $context, $subContext)
    {
        $enabledCodes = \XF::options()->realTimeChatEnabledBbCodes;
        $customCodes = \XF::app()->bbCode()['custom'];

        foreach ($enabledCodes AS $code => $isEnable) 
        {
            if (!$isEnable && $ruleSet->getTag($code))
            {
                $ruleSet->removeTag($code);
            }
        }

        foreach ($customCodes AS $code => $tag) 
        {
            $ruleSet->removeTag($code);
        }
    }

    public static function templaterFunc()
    {
        return \XF::$versionId >= 2010370 ? 'func' : 'fn';
    }
}