<?php

namespace BS\RealTimeChat\Service\Message;

use BS\RealTimeChat\Entity\Message;
use LogicException;
use XF\App;
use XF\Service\AbstractService;

class CommandRenderer extends AbstractService
{
    protected $renderers;
    protected $message;

    public function __construct(App $app, Message $message)
    {
        parent::__construct($app);
        
        $this->message = $message;
    }

    public function render()
    {
        $registry = $this->app->registry();

        $commands = $registry->get('bsRtcCommands');
        $messageText = $this->message->message;

        foreach ($commands AS $command) 
        {
            if (preg_match($command['command_regex'], $messageText))
            {
                return $this->getCommandRenderer($command['command_class'])->render();
            }
        }

        return true;
    }

    protected function getCommandRenderer($identifier)
    {
        $app = $this->app;
        
        if (isset($this->renderers[$identifier]))
        {
            $this->renderers[$identifier]->setMessage($this->message);
            return $this->renderers[$identifier];
        }

        $rendererClass = $app->extension()->extendClass($identifier, '\BS\RealTimeChat\ChatCommand\AbstractCommand');
        if (!$rendererClass || !class_exists($rendererClass))
        {
            throw new LogicException("Could not find chat command renderer '$rendererClass' for '$identifier'");
        }

        $renderer = new $rendererClass($this->app, $this->message);
        $this->renderers[$identifier] = $renderer;

        return $renderer;
    }
}