<?php

namespace BS\RealTimeChat;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Alter;
use XF\Db\Schema\Create;

class Setup extends AbstractSetup
{
	use StepRunnerInstallTrait;
	use StepRunnerUpgradeTrait;
	use StepRunnerUninstallTrait;

    // ################################ INSTALLATION ####################

    public function installStep1()
    {
        $sm = $this->schemaManager();

        foreach ($this->getTables() AS $tableName => $closure)
        {
            $sm->createTable($tableName, $closure);
        }
    }
    
    public function installStep2()
    {
        foreach ($this->getWidgets() AS $widgetKey => $widget) 
        {
            $this->createWidget($widgetKey, $widget['definition_id'], $widget['config'], $widget['title']);
        }
    }

    public function installStep3()
    {
        $db = $this->db();

        foreach ($this->getCommands() AS $commandId => $command) 
        {
            $insert = $command + [
                'command_id' => $commandId,
                'addon_id' => 'BS/RealTimeChat'
            ];

            $db->insert('xf_bs_chat_command', $insert);
        }
    }

    // ############################### UPGRADE 1.0.0 BETA 2 #########################

    public function upgrade1000032Step1()
    {
        $this->schemaManager()->createTable('xf_chat_banned', function(Create $table)
        {
            $table->addColumn('banned_id', 'int')->autoIncrement();
            $table->addColumn('user_id', 'int');
            $table->addColumn('banned_user_id', 'int');
            $table->addColumn('date', 'int');
            $table->addColumn('unban_date', 'int');
            $table->addColumn('reason', 'varchar', 300)->nullable();
            $table->addKey('user_id');
            $table->addKey('banned_user_id');
        });
    }

    // ############################### UPGRADE 1.0.0 ############################

    public function upgrade1000070Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_chat', function(Alter $table)
        {
            $table->renameTo('xf_bs_chat');
            $table->renameColumn('date', 'message_date');
        });

        $sm->alterTable('xf_chat_banned', function(Alter $table)
        {
            $table->renameTo('xf_bs_chat_banned');
        });
    }

    public function upgrade1000070Step2()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_bs_chat', function(Alter $table)
        {
            $table->changeColumn('message_date', 'bigint', 20);
            $table->addKey('message_date');
        });
    }

    public function upgrade1000070Step3()
    {
        $this->schemaManager()->createTable('xf_bs_chat_command', function(Create $table)
        {
            $table->addColumn('command_id', 'varbinary', 25)->primaryKey();
            $table->addColumn('command_regex', 'text');
            $table->addColumn('command_class', 'varchar', 300);
            $table->addColumn('active', 'tinyint', 3)->setDefault(1);
            $table->addColumn('addon_id', 'varbinary', 50);
            $table->addKey('active');
            $table->addKey('addon_id');
        });
    }

    public function upgrade1000070Step4()
    {
        $db = $this->db();

        $commands = [
            'clear' => [
                'command_class' => 'BS\RealTimeChat\ChatCommand\Clear',
                'command_regex' => '/^\/clear$/i',
                'active' => true
            ],
            'clear_user_messages' => [
                'command_class' => 'BS\RealTimeChat\ChatCommand\ClearUser',
                'command_regex' => '/^\/clear\s\[user=([0-9]*)]@.*\[\/user]$/iu',
                'active' => true
            ]
        ];

        foreach ($commands AS $commandId => $command) 
        {
            $insert = $command + [
                'command_id' => $commandId,
                'addon_id' => 'BS/RealTimeChat'
            ];

            $db->insert('xf_bs_chat_command', $insert);
        }
    }

    // ############################### UPGRADE 1.0.2 ############################

    public function upgrade1000270Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_bs_chat', function(Alter $table)
        {
            $table->addColumn('to_user_id', 'int')->setDefault(0);
            $table->addColumn('pm_user_id', 'int')->setDefault(0);
            $table->addKey('to_user_id');
            $table->addKey('pm_user_id');
        });
    }

    public function upgrade1000270Step2()
    {
        $this->db()->update('xf_bs_chat_command', [
            'command_regex' => '/^\/clear\s\[user=([0-9]*)].*\[\/user]$/iu'
        ], 'command_id = \'clear_user_messages\'');
    }

    public function upgrade1000270Step3()
    {
        $db = $this->db();

        $commands = [
            'to' => [
               'command_class' => 'BS\RealTimeChat\ChatCommand\To',
                'command_regex' => '/^\/to\s\[user=([0-9]*)].*\[\/user]/iu',
                'active' => true
            ],
            'pm' => [
                'command_class' => 'BS\RealTimeChat\ChatCommand\Pm',
                'command_regex' => '/^\/pm\s\[user=([0-9]*)].*\[\/user]/iu',
                'active' => true
            ]
        ];

        foreach ($commands AS $commandId => $command) 
        {
            $insert = $command + [
                'command_id' => $commandId,
                'addon_id' => 'BS/RealTimeChat'
            ];

            $db->insert('xf_bs_chat_command', $insert);
        }
    }

    // ############################### UPGRADE 1.1.0 ############################

    public function upgrade1010070Step1()
    {
        $this->schemaManager()->createTable('xf_bs_chat_long_polling_event', function (Create $table)
        {
            $table->addColumn('event_id', 'int')->autoIncrement();
            $table->addColumn('event_time', 'int')->setDefault(0);
            $table->addColumn('event_type', 'varchar', 30);
            $table->addColumn('event_data', 'mediumblob')->nullable();
            $table->addColumn('room_ids', 'blob');
            $table->addKey('event_time');
        });
    }

    // ############################### UNINSTALL #########################
    
    public function uninstallStep1(array $stepParams = [])
    {
        $sm = $this->schemaManager();

        foreach (array_keys($this->getTables()) AS $tableName)
        {
            $sm->dropTable($tableName);
        }
    }

    // ############################### POST INSTALL #########################

    public function postInstall(array &$stateChanges)
    {
        $this->app->repository('BS\RealTimeChat:Command')->rebuildCommandsCache();
    }

    // ############################### POST UPGRADE #########################

    public function postUpgrade($previousVersion, array &$stateChanges)
    {
        $this->app->repository('BS\RealTimeChat:Command')->rebuildCommandsCache();
    }

    // ############################# TABLE / DATA DEFINITIONS ##############################

    protected function getTables()
    {
        $tables = [];

        $tables['xf_bs_chat'] = function(Create $table)
        {
            $table->addColumn('message_id', 'int')->autoIncrement();
            $table->addColumn('user_id', 'int');
            $table->addColumn('message', 'text');
            $table->addColumn('message_date', 'bigint', 20);
            $table->addColumn('to_user_id', 'int')->setDefault(0);
            $table->addColumn('pm_user_id', 'int')->setDefault(0);
            $table->addKey('user_id');
            $table->addKey('message_date');
            $table->addKey('to_user_id');
            $table->addKey('pm_user_id');
        };

        $tables['xf_bs_chat_banned'] = function(Create $table)
        {
            $table->addColumn('banned_id', 'int')->autoIncrement();
            $table->addColumn('user_id', 'int');
            $table->addColumn('banned_user_id', 'int');
            $table->addColumn('date', 'int');
            $table->addColumn('unban_date', 'int');
            $table->addColumn('reason', 'varchar', 300)->nullable();
            $table->addKey('user_id');
            $table->addKey('banned_user_id');
        };

        $tables['xf_bs_chat_command'] = function(Create $table)
        {
            $table->addColumn('command_id', 'varbinary', 25)->primaryKey();
            $table->addColumn('command_regex', 'text');
            $table->addColumn('command_class', 'varchar', 300);
            $table->addColumn('active', 'tinyint', 3)->setDefault(1);
            $table->addColumn('addon_id', 'varbinary', 50);
            $table->addKey('active');
            $table->addKey('addon_id');
        };

        $tables['xf_bs_chat_long_polling_event'] = function (Create $table)
        {
            $table->addColumn('event_id', 'int')->autoIncrement();
            $table->addColumn('event_time', 'int')->setDefault(0);
            $table->addColumn('event_type', 'varchar', 30);
            $table->addColumn('event_data', 'mediumblob')->nullable();
            $table->addColumn('room_ids', 'blob');
            $table->addKey('event_time');
        };

        return $tables;
    }

    protected function getWidgets()
    {
        $widgets = [];

        $widgets['real_time_chat'] = [
            'definition_id' => 'real_time_chat',
            'config' => [
                'positions' => [
                    'forum_list_above_nodes' => 10
                ]
            ],
            'title' => ''
        ];

        return $widgets;
    }

    protected function getCommands()
    {
        $commands = [];

        $commands['clear'] = [
            'command_class' => 'BS\RealTimeChat\ChatCommand\Clear',
            'command_regex' => '/^\/clear$/i',
            'active' => true
        ];

        $commands['clear_user_messages'] = [
            'command_class' => 'BS\RealTimeChat\ChatCommand\ClearUser',
            'command_regex' => '/^\/clear\s\[user=([0-9]*)].*\[\/user]$/iu',
            'active' => true
        ];

        $commands['to'] = [
            'command_class' => 'BS\RealTimeChat\ChatCommand\To',
            'command_regex' => '/^\/to\s\[user=([0-9]*)].*\[\/user]/iu',
            'active' => true
        ];

        $commands['pm'] = [
            'command_class' => 'BS\RealTimeChat\ChatCommand\Pm',
            'command_regex' => '/^\/pm\s\[user=([0-9]*)].*\[\/user]/iu',
            'active' => true
        ];

        return $commands;
    }
}