<?php

namespace BS\RealTimeChat\Workerman;

use Workerman\Worker;
use PHPSocketIO\SocketIO;
use PHPSocketIO\ChannelAdapter;
use Channel\Server;

class ServerIO extends SocketIO
{
    protected static $events = [
        \BS\RealTimeChat\Workerman\Events\Auth::class
    ];

    public static function start()
    {
        $options = \XF::options();

        $port = $options->realTimeChatPort;
        $channelAdapterPort = $options->realTimeChatChannelAdapterPort;
        $ioOptions = [];

        switch ($options->realTimeChatClientCommunication['communication'])
        {
            case 'direct':
                if ($options->realTimeChatClientCommunication['sslCertPath'] && $options->realTimeChatClientCommunication['sslKeyPath'])
                {
                    $ioOptions = [
                        'ssl' => [
                            'local_cert'  => $options->realTimeChatClientCommunication['sslCertPath'],
                            'local_pk'    => $options->realTimeChatClientCommunication['sslKeyPath'],
                            'verify_peer' => false
                        ]
                    ];
                }
                break;

            case 'proxying':
                break;

            default:
                return self::writeln('Unknown communication type.');
                break;
        }

        if (! $channelAdapterPort)
        {
            return self::writeln('Work without channel adapter is impossible.');
        }

        $server = new self($port, $ioOptions);
        $server->on('workerStart', function() use ($server, $channelAdapterPort, $port)
        {
            ChannelAdapter::$port = $channelAdapterPort;
            $server->adapter('\PHPSocketIO\ChannelAdapter');

            self::writeln("Created server on port {$port}");
        });

        new Server('127.0.0.1', $channelAdapterPort);

        if (extension_loaded('ev'))
        {
            Worker::$eventLoopClass = '\Workerman\Events\Ev';
        }

        foreach (self::$events as $event)
        {
            new $event($server);
        }

        \XF::db()->closeConnection();

        Worker::runAll();
    }

    protected static function writeln($msg, $decorated = false)
    {
        Worker::safeEcho($msg . "\n", $decorated);
    }
}